﻿using System;
using System.ComponentModel;
using System.Runtime.InteropServices;
using IpPbx.Client.AudioDevices.SDK;
using IpPbx.Client.Devices.Audio.Plugins.Sample.UI.WinForms;

namespace IpPbx.Client.Devices.Audio.Plugins.Sample
{
    /// <summary>
    /// This is simple test audio device implementation. It behaves
    /// like a headset to Client Line Manager. Additionally it opens
    /// a little UI which offers the oppurtunity to fire events to 
    /// Client Line Manager application.
    /// </summary>
    [ComVisible(true)]
    [Guid("E499D5CA-673F-4625-817D-8D2FE30A6934")]
    [ClassInterface(ClassInterfaceType.None)]
    [ComSourceInterfaces(typeof(_IAudioDeviceEvents))]
    public class MyAudioDevice : AudioDevice
    {
        #region fields
        /// <summary>
        /// This name is displayed in SwyxIt! UI.
        /// </summary>
        private string name = string.Empty;

        /// <summary>
        /// This name is displayed in SwyxIt! UI - it contains the "real"
        /// audio device name from the registry configuration.
        /// </summary>
        private string deviceName = string.Empty;

        /// <summary>
        /// This is the render audio control for that test device.
        /// </summary>
        private IAudioVolumeControl renderControl = null;

        /// <summary>
        /// This is the capture audio control for that test device.
        /// </summary>
        private IAudioVolumeControl captureControl = null;

        /// <summary>
        /// This is an abstract UI communication interface.
        /// </summary>
        private IAudioDeviceSampleGUI gui = null;

        /// <summary>
        /// This is used to run the UI in its own thread.
        /// </summary>
        private BackgroundWorker uiBackgroundWorker = null;

        #endregion

        #region constructor

        public MyAudioDevice(string name, string deviceName, IAudioVolumeControl render, IAudioVolumeControl capture)
        {
            // Set members.
            this.name = string.IsNullOrEmpty(name) ? "Sample Device" : name;
            this.deviceName = deviceName;
            this.renderControl = render;
            this.captureControl = capture;

            // Activate UI window.
            this.uiBackgroundWorker = new BackgroundWorker();
            this.uiBackgroundWorker.DoWork += new DoWorkEventHandler(bgw_DoWork);
            this.uiBackgroundWorker.RunWorkerAsync();
        }

        #endregion

        #region overrides

        /// <inheritdoc />
        public override string FriendlyName
        {
            get
            {
                return string.Format("{0} ({1})",
                    this.name,
                    this.deviceName);
            }
        }

        /// <inheritdoc />
        public override ushort GetDefaultVolume(AudioMode am, SoundChannel sc)
        {
            // 50% default volume for our sample device for all sound channels.
            return 50;
        }

        /// <inheritdoc />
        public override string GetSoundChannelID(SoundChannel sc)
        {
            if (this.captureControl != null && sc == SoundChannel.SoundChannelCapture)
                return this.captureControl.GUIDAsString;
            else if (this.renderControl != null && sc == SoundChannel.SoundChannelRender)
                return this.renderControl.GUIDAsString;
            return string.Empty; // No sound channel ID found for that request.
        }

        /// <inheritdoc />
        public override bool IsChannelUsedByDevice(IAudioVolumeControl ctrl)
        {
            // Simply checks whether the requested Audio volume control is used by our test device.
            return (this.renderControl != null && ctrl.GUID.Equals(this.renderControl.GUID)) || 
                (this.captureControl != null && ctrl.GUID.Equals(this.captureControl.GUID));
        }

        /// <inheritdoc />
        public override bool IsSoundChannelSupported(AudioMode am, SoundChannel sc)
        {
            // Simply checks whether our test device supports the requested sound channel.
            // For our sample: We just return true, if a headset configuration is requested.
            return (am == AudioMode.AudioModeHeadset) &&
                        ((sc == SoundChannel.SoundChannelCapture) || 
                         (sc == SoundChannel.SoundChannelRender));
        }

        /// <inheritdoc />
        public override AudioModeField PreferredAudioModes
        {
            get
            { 
                // We prefer a headset configuration for that device.
                return AudioModeField.Headset;
            }
        }

        /// <inheritdoc />
        public override AudioModeField SupportedAudioModes
        {
            get
            {
                // This is a headset device.
                return AudioModeField.Headset;
            }
        }

        /// <summary>
        /// We override the Dispose method here because we want to close
        /// our sample UI window.
        /// </summary>
        public new void Dispose()
        {
            try
            {
                // Close sample UI
                try
                {
                    this.gui.CloseWindow();
                }
                finally
                {
                    this.gui = null;
                    this.uiBackgroundWorker.Dispose();
                }
            }
            catch (Exception ex)
            {
                TraceException(ex);
            }
            finally
            {
                this.uiBackgroundWorker = null;
                base.Dispose();
            }
        }

        /// <summary>
        /// This is called by the SwyxIt! UI when the user pressed the
        /// hook off/hook on button.
        /// </summary>
        public override void SetHookedOff(bool hookedOff)
        {
            this.gui.SetHookedOff(hookedOff);
        }

        /// <summary>
        /// This is called by SwyxIt! UI when the user pressed the
        /// mute button.
        /// </summary>
        public override void SetMuted(bool muted)
        {
            if (this.captureControl != null)
            {
                this.captureControl.Mute = muted ? 1 : 0;
            }
            this.gui.SetMuted(muted);
        }

        /// <inheritdoc />
        public override void SetDisplayCallerInfo(string bstrCallerInfo)
        {
            this.gui.SetCallerText(bstrCallerInfo);
        }

        /// <inheritdoc />
        public override void SetRinger(bool ringing, bool isRingingDevice)
        {
            this.gui.SetRinger(ringing);
        }

        #endregion

        #region event handler

        /// <summary>
        /// This is used to run the UI window in its own thread.
        /// </summary>
        private void bgw_DoWork(object sender, DoWorkEventArgs e)
        {
            try
            {
                this.gui = new frmDemoUI();
                this.gui.OnHook += new Sample.OnHookHandler(gui_OnHook);
                this.gui.OnMute += new Sample.OnMuteHandler(gui_OnMute);
                this.gui.OnRejectCall += new Sample.OnRejectCallHandler(gui_OnRejectCall);
                this.gui.AudioDeviceName = string.Format("{0} ({1})",
                    this.name, this.deviceName);
                this.gui.ShowWindow();
            }
            catch (Exception ex)
            {
                TraceException(ex);
            }
        }

        /// <summary>
        /// This is called when the UI wants to reject an incoming call.
        /// </summary>
        private void gui_OnRejectCall()
        {
            FireOnRejectCall();
        }

        /// <summary>
        /// Fired by UI when user wants device to be muted/unmuted.
        /// </summary>
        private void gui_OnMute(bool muted)
        {
            FireOnMute(muted ? 1 : 0);
        }

        /// <summary>
        /// Fired by UI when user wants device to be hooked off/hooked on.
        /// </summary>
        private void gui_OnHook(bool hookOff)
        {
            FireOnHook(hookOff ? 1 : 0,
                AudioMode.AudioModeHeadset);
        }

        #endregion
    }
}
