﻿using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using IpPbx.Client.AudioDevices.SDK;
using Microsoft.Win32;
using System.Windows.Forms;

namespace IpPbx.Client.Devices.Audio.Plugins.Sample
{
    /// <summary>
    /// This is the main plugin implementation. The AudioDeviceCollection
    /// manages the instances of the AudioDevices we are using and provides
    /// them to Client Line Manager.
    /// </summary>
    [ComVisible(true)]
    [Guid("606374E2-44D1-45CC-96CF-60200B433FF1")]
    [ClassInterface(ClassInterfaceType.None)]
    public class MyAudioDeviceCollection : AudioDeviceCollection
    {
        #region fields

        /// <summary>
        /// List which contains AudioDevice instances for this plugin
        /// </summary>
        private List<IAudioDevice> audioDeviceList =
            new List<IAudioDevice>();

        /// <summary>
        /// This list contains all test device configurations from the registry key.
        /// </summary>
        List<TestDeviceConfig> deviceConfigList = null;

        #endregion

        static MyAudioDeviceCollection()
        {
            // Just to make our dialog look a bit nicer...
            Application.EnableVisualStyles();
        }

        #region helper methods

        /// <summary>
        /// This helper method just reads out the audio devices which belongs to the
        /// device with a specific friendly name.
        /// Hint: This method only works for devices with one render and one capture audio
        /// control.
        /// </summary>
        /// <param name="name">The friendly name of the requested device.</param>
        /// <param name="render">The render audio control of the device.</param>
        /// <param name="capture">The capture audio control of the device.</param>
        /// <returns>True if either a render channel or capture channel (or both channels) of the device with the requested name were found.</returns>
        public static bool GetAudioControlsByName(string name, out IAudioVolumeControl render, out IAudioVolumeControl capture)
        {
            bool res = false;

            render = null;
            capture = null;

            AudioVolumeControlCollection coll = new AudioVolumeControlCollection();
            try
            {
                // Find audio controls by name (FriendlyName will be used internally to identify the controls)
                coll.Initialize(name);
                try
                {
                    int count = coll.Count;
                    for (uint n = 0; n < count; n++)
                    {
                        IAudioVolumeControl ctrl = coll.GetAudioVolumeControl(n) as IAudioVolumeControl;
                        if (ctrl != null)
                        {
                            if (ctrl.DataFlow == AudioDeviceDataFlow.DataFlowRender)
                            {
                                res = true;
                                render = ctrl;
                            }
                            else if (ctrl.DataFlow == AudioDeviceDataFlow.DataFlowCapture)
                            {
                                res = true;
                                capture = ctrl;
                            }
                        }
                    }
                }
                finally
                {
                    coll.UnInitialize();
                }
            }
            finally
            {
                Marshal.ReleaseComObject(coll);
                coll = null;
            }

            return res;
        }

        /// <summary>
        /// This method reads out the test device configuration from the registry.
        /// </summary>
        /// <param name="regKey">The registry key where we want to read the configuration from.</param>
        /// <returns>A list of test device configurations.</returns>
        private List<TestDeviceConfig> ReadTestDeviceConfiguration(string regKey)
        {
            List<TestDeviceConfig> res = new List<TestDeviceConfig>();
            try
            {
                using (RegistryKey key = Registry.LocalMachine.OpenSubKey(regKey))
                {
                    string[] subKeys = key.GetSubKeyNames();
                    if (subKeys != null && subKeys.Length > 0)
                    {
                        foreach (string sk in subKeys)
                        {
                            try
                            {
                                using (RegistryKey subKey = key.OpenSubKey(sk))
                                {
                                    object audioNameVal = subKey.GetValue("AudioDeviceName");
                                    if (audioNameVal == null)
                                    {
                                        throw new Exception(
                                            string.Format("Unable to find AudioDeviceName value for device config \"{0}\".", subKey.Name));
                                    }

                                    res.Add(new TestDeviceConfig()
                                    {
                                        Name = sk,
                                        AudioDeviceName = audioNameVal.ToString()
                                    });
                                }
                            }
                            catch (Exception itemEx)
                            {
                                Trace("* Exception: " + itemEx.Message);
                                continue;
                            }
                        }
                    }
                }
            }
            catch
            {
                res.Clear();
            }

            return res;
        }

        #endregion

        #region overrides

        /// <summary>
        /// Just returning the count of our audio devices.
        /// </summary>
        public override int Count
        {
            get
            {
                return this.audioDeviceList.Count;
            }
        }

        /// <inheritdoc />
        public override System.Collections.IEnumerator GetEnumerator()
        {
            return this.audioDeviceList.GetEnumerator();
        }

        /// <inheritdoc />
        public override void Initialize(string bstrRegKey)
        {
            try
            {
                // Read registry parameters for sample devices configuration
                this.deviceConfigList = ReadTestDeviceConfiguration(bstrRegKey);
                Trace(string.Format("Read {0} device configuration items.", deviceConfigList.Count));

                // Run through our device configuration and find the appropriate audio controls.
                IAudioVolumeControl render;
                IAudioVolumeControl capture;
                foreach(TestDeviceConfig cfg in this.deviceConfigList)
                {
                    if (GetAudioControlsByName(cfg.AudioDeviceName, out render, out capture))
                    {
                        this.audioDeviceList.Add(
                            new MyAudioDevice(cfg.Name, cfg.AudioDeviceName, render, capture));
                    }
                }
            }
            catch (Exception ex)
            {
                TraceException(ex);
            }
        }

        /// <inheritdoc />
        public override object Item(object index)
        {
            object res = null;

            try
            {
                if (this.audioDeviceList.Count > 0)
                {
                    int i = Convert.ToInt32(index);
                    // COM sends 1-based indices here, so:
                    i--;
                    if (i < 0 || i >= this.audioDeviceList.Count)
                    {
                        Trace(string.Format("Requested i: {0}, Count: {1} -> throwing ArgumentOutOfRangeException.",
                            i,
                            this.audioDeviceList.Count));
                        throw new ArgumentOutOfRangeException();
                    }

                    res = this.audioDeviceList[i];
                }
            }
            catch (Exception ex)
            {
                TraceException(ex);
            }

            return res;
        }

        /// <summary>
        /// A friendly name of this plugin.
        /// </summary>
        public override string PluginFriendlyName
        {
            get
            { 
                return "Audio Device SDK Sample PlugIn"; 
            }
        }

        /// <summary>
        /// The version of this plugin.
        /// </summary>
        public override string PluginVersion
        {
            get 
            { 
                return "1.0.0.0"; 
            }
        }

        /// <summary>
        /// This is called, when Client Line Manager unloads this plugin.
        /// </summary>
        public override void UnInitialize()
        {
            // Nothing to be done for our demo code. :-)
        }

        #endregion
    }
}
